@@ -29,6 +29,7 @@ from ironic.drivers import base
 from ironic.drivers.modules import deploy_utils
 from ironic.drivers.modules.drac import common as drac_common
 from ironic.drivers.modules.drac import job as drac_job
+from ironic.drivers.modules.redfish import bios as redfish_bios
 from ironic import objects
 
 drac_exceptions = importutils.try_import('dracclient.exceptions')
@@ -38,6 +39,16 @@ LOG = logging.getLogger(__name__)
 METRICS = metrics_utils.get_metrics_logger(__name__)
 
 
+class DracRedfishBIOS(redfish_bios.RedfishBIOS):
+    """iDRAC Redfish interface for BIOS settings-related actions.
+
+    Presently, this class entirely defers to its base class, a generic,
+    vendor-independent Redfish interface. Future resolution of Dell EMC-
+    specific incompatibilities and introduction of vendor value added
+    should be implemented by this class.
+    """
+
+
 class DracWSManBIOS(base.BIOSInterface):
     """BIOSInterface Implementation for iDRAC."""
 
