@@ -87,7 +87,7 @@ COMMON_PROPERTIES.update(OPTIONAL_PROPERTIES)
 
 
 def _is_image_href_ordinary_file_name(image_href):
-    """Check if image_href is a ordinary file name.
+    """Check if image_href is an ordinary file name.
 
     This method judges if image_href is an ordinary file name or not,
     which is a file supposed to be stored in share file system.
@@ -580,8 +580,6 @@ class IRMCVolumeBootMixIn(object):
     used by VIOM (Virtual I/O Manager) library of SCCI client.
     """
 
-    supported = False
-
     def _validate_volume_boot(self, task):
         """Validate information for volume boot with this interface.
 
@@ -900,8 +898,6 @@ class IRMCVolumeBootMixIn(object):
 class IRMCVirtualMediaBoot(base.BootInterface, IRMCVolumeBootMixIn):
     """iRMC Virtual Media boot-related actions."""
 
-    supported = False
-
     capabilities = ['iscsi_volume_boot', 'fibre_channel_volume_boot']
 
     def __init__(self):
@@ -1105,8 +1101,6 @@ class IRMCVirtualMediaBoot(base.BootInterface, IRMCVolumeBootMixIn):
 class IRMCPXEBoot(pxe.PXEBoot):
     """iRMC PXE boot."""
 
-    supported = False
-
     @METRICS.timer('IRMCPXEBoot.prepare_ramdisk')
     def prepare_ramdisk(self, task, ramdisk_params):
         """Prepares the boot of Ironic ramdisk using PXE.
