@@ -43,7 +43,8 @@ class MockResponse(object):
 
 
 class MockCommandStatus(MockResponse):
-    def __init__(self, status, name='fake', error=None):
+    def __init__(self, status, name='fake', error=None,
+                 status_code=http_client.OK):
         super().__init__({
             'commands': [
                 {'command_name': name,
@@ -54,6 +55,12 @@ class MockCommandStatus(MockResponse):
         })
 
 
+class MockFault(MockResponse):
+    def __init__(self, faultstring, status_code=http_client.BAD_REQUEST):
+        super().__init__({'faultstring': faultstring},
+                         status_code=status_code)
+
+
 class MockNode(object):
     def __init__(self):
         self.uuid = 'uuid'
@@ -73,6 +80,9 @@ class MockNode(object):
             'driver_info': self.driver_info,
         }
 
+    def save(self):
+        pass
+
 
 class TestAgentClient(base.TestCase):
     def setUp(self):
@@ -227,7 +237,8 @@ class TestAgentClient(base.TestCase):
             timeout=60,
             verify=True)
 
-    def test__command_verify(self):
+    @mock.patch('os.path.exists', autospec=True, return_value=True)
+    def test__command_verify(self, mock_exists):
         response_data = {'status': 'ok'}
         self.client.session.post.return_value = MockResponse(response_data)
         method = 'standby.run_image'
@@ -248,6 +259,86 @@ class TestAgentClient(base.TestCase):
             timeout=60,
             verify='/path/to/agent.crt')
 
+    @mock.patch('os.path.exists', autospec=True, return_value=True)
+    def test__command_verify_internal(self, mock_exists):
+        response_data = {'status': 'ok'}
+        self.client.session.post.return_value = MockResponse(response_data)
+        method = 'standby.run_image'
+        image_info = {'image_id': 'test_image'}
+        params = {'image_info': image_info}
+
+        self.node.driver_info['agent_verify_ca'] = True
+        self.node.driver_internal_info['agent_verify_ca'] = '/path/to/crt'
+
+        url = self.client._get_command_url(self.node)
+        body = self.client._get_command_body(method, params)
+
+        response = self.client._command(self.node, method, params)
+        self.assertEqual(response, response_data)
+        self.client.session.post.assert_called_once_with(
+            url,
+            data=body,
+            params={'wait': 'false'},
+            timeout=60,
+            verify='/path/to/crt')
+
+    @mock.patch('os.path.exists', autospec=True, return_value=True)
+    def test__command_verify_config(self, mock_exists):
+        response_data = {'status': 'ok'}
+        self.client.session.post.return_value = MockResponse(response_data)
+        method = 'standby.run_image'
+        image_info = {'image_id': 'test_image'}
+        params = {'image_info': image_info}
+
+        self.config(verify_ca='/path/to/crt', group='agent')
+
+        url = self.client._get_command_url(self.node)
+        body = self.client._get_command_body(method, params)
+
+        response = self.client._command(self.node, method, params)
+        self.assertEqual(response, response_data)
+        self.client.session.post.assert_called_once_with(
+            url,
+            data=body,
+            params={'wait': 'false'},
+            timeout=60,
+            verify='/path/to/crt')
+
+    @mock.patch('os.path.exists', autospec=True, return_value=True)
+    def test__command_verify_disable(self, mock_exists):
+        response_data = {'status': 'ok'}
+        self.client.session.post.return_value = MockResponse(response_data)
+        method = 'standby.run_image'
+        image_info = {'image_id': 'test_image'}
+        params = {'image_info': image_info}
+
+        self.config(verify_ca='False', group='agent')
+
+        url = self.client._get_command_url(self.node)
+        body = self.client._get_command_body(method, params)
+
+        response = self.client._command(self.node, method, params)
+        self.assertEqual(response, response_data)
+        self.client.session.post.assert_called_once_with(
+            url,
+            data=body,
+            params={'wait': 'false'},
+            timeout=60,
+            verify=False)
+
+    @mock.patch('os.path.exists', autospec=True, return_value=False)
+    def test__command_verify_invalid_file(self, mock_exists):
+        response_data = {'status': 'ok'}
+        self.client.session.post.return_value = MockResponse(response_data)
+        method = 'standby.run_image'
+        image_info = {'image_id': 'test_image'}
+        params = {'image_info': image_info}
+
+        self.config(verify_ca='/path/to/crt', group='agent')
+
+        self.assertRaises(exception.InvalidParameterValue,
+                          self.client._command, self.node, method, params)
+
     @mock.patch('time.sleep', lambda seconds: None)
     def test__command_poll(self):
         response_data = {'status': 'ok'}
@@ -312,7 +403,8 @@ class TestAgentClient(base.TestCase):
                           retry_connection=False)
         self.assertEqual(1, self.client.session.get.call_count)
 
-    def test_get_commands_status_verify(self):
+    @mock.patch('os.path.exists', autospec=True, return_value=True)
+    def test_get_commands_status_verify(self, mock_exists):
         self.node.driver_info['agent_verify_ca'] = '/path/to/agent.crt'
 
         with mock.patch.object(self.client.session, 'get',
@@ -605,6 +697,33 @@ class TestAgentClientAttempts(base.TestCase):
             timeout=60,
             verify=True)
 
+    @mock.patch.object(retrying.time, 'sleep', autospec=True)
+    def test__command_fail_agent_token_required(self, mock_sleep):
+        mock_sleep.return_value = None
+        error = 'Unknown Argument: "agent_token"'
+        method = 'standby.run_image'
+        image_info = {'image_id': 'test_image'}
+        params = {'image_info': image_info}
+        i_info = self.node.driver_internal_info
+        i_info['agent_secret_token'] = 'meowmeowmeow'
+        self.client.session.post.side_effect = [
+            MockFault(error)
+        ]
+
+        self.assertRaises(exception.AgentAPIError,
+                          self.client._command,
+                          self.node, method, params)
+        self.assertEqual(1, self.client.session.post.call_count)
+        self.client.session.post.assert_called_with(
+            self.client._get_command_url(self.node),
+            data=self.client._get_command_body(method, params),
+            params={'wait': 'false', 'agent_token': 'meowmeowmeow'},
+            timeout=60,
+            verify=True)
+        self.assertEqual(
+            'meowmeowmeow',
+            self.node.driver_internal_info.get('agent_secret_token'))
+
     @mock.patch.object(retrying.time, 'sleep', autospec=True)
     def test__command_succeed_after_one_timeout(self, mock_sleep):
         mock_sleep.return_value = None
