@@ -15,8 +15,8 @@
 
 import datetime
 import os
+from unittest import mock
 
-import mock
 from oslo_config import cfg
 from oslo_utils import timeutils
 
